"""
This module is designed to be included from Sphinx conf.py
configuration files to set reasonable defaults for STScI projects.

It should be imported from the top of your `conf.py` as::

  from stsci.sphinxext.conf import *

The "extensions" line in your `conf.py` should also be modified so it
doesn't override the extensions defined here.  For example, change::

  extensions = ['sphinx.ext.autodoc']

to::

  extensions += ['sphinx.ext.autodoc']

Also, comment out the 'html_theme' line.
"""

import glob
import os
import sys

# Store the directory that this file is in so we can get at our data
__dir__ = os.path.abspath(os.path.dirname(__file__))

# In order to get Sphinx to import extensions in this directory
sys.path.insert(0, __dir__)

primary_domain = 'py'

try:
    import numpydoc
except ImportError:
    print >> sys.stderr, ('numpydoc and Numpy are required to build this '
                          'documentation')
    sys.exit(1)

try:
    import matplotlib.sphinxext
except ImportError:
    print >> sys.stderr, ('matplotlib, and specifically matplotlib.sphinxext '
                          'are required to build this documentation')
    sys.exit(1)

# A list of standard extensions
extensions = [
    'narrow_field_lists',      # Create field lists that don't waste
                               # so much horizontal space
    'abstract',                # Support an abstract directive
#    'parameter_anchor',        # Add HTML anchors to parameter names
    'sphinx.ext.autodoc',      # Extract documentation from docstrings
    'sphinx.ext.intersphinx',  # Link to other Sphinx documentation trees
    'sphinx.ext.pngmath',      # Render math as images
    'sphinx.ext.inheritance_diagram',
    'sphinx.ext.autosummary',  # (Required by numpydoc)
    'numpydoc',                # Support the Numpy docstring format
    'matplotlib.sphinxext.plot_directive', # matplotlib plots
    'matplotlib.sphinxext.only_directives'
    ]

# Class documentation should contain *both* the class docstring and
# the __init__ docstring
autoclass_content = "both"

autodoc_member_order = "groupwise"

# Set the default role to the "smart" one
default_role = 'obj'

# Support linking directly to the Python standard library and Numpy
intersphinx_mapping = {
    'http://docs.python.org/': None,
    'http://docs.scipy.org/doc/numpy': None,
    'http://matplotlib.org/': None,
    }

# Don't show summaries of the members in each class along with the
# class' docstring
numpydoc_show_class_members = False

# ----------------------------------------------------------------------
# HTML OPTIONS
# ----------------------------------------------------------------------

# Override the default HTML theme
html_theme_path = [__dir__]
html_theme = 'stsci_sphinx_theme'
html_logo = os.path.join(
    __dir__, 'stsci_sphinx_theme', 'static', 'stsci_logo.png')

# This is required to remove the superfluous np module index added by numpydoc
html_domain_indices = ['py-modindex']

# ----------------------------------------------------------------------
# LATEX OPTIONS
# ----------------------------------------------------------------------

# Additional stuff for the LaTeX preamble.
latex_preamble = r'''
\usepackage{enumitem}
\setlistdepth{15}

\usepackage{amsmath}
\DeclareUnicodeCharacter{00A0}{\nobreakspace}

% In the parameters section, place a newline after the Parameters
% header
\usepackage{expdlist}
\let\latexdescription=\description
\def\description{\latexdescription{}{} \breaklabel}

% Make Examples/etc section headers smaller and more compact
\makeatletter
\titleformat{\paragraph}{\normalsize\py@HeaderFamily}%
            {\py@TitleColor}{0em}{\py@TitleColor}{\py@NormalColor}
\titlespacing*{\paragraph}{0pt}{1ex}{0pt}

% Fix footer/header
\@ifundefined{chaptermark}{%
\newcommand{\chaptermark}[1]{\markboth{\MakeUppercase{\thechapter.\ #1}}{}}
}{%
\renewcommand{\chaptermark}[1]{\markboth{\MakeUppercase{\thechapter.\ #1}}{}}
}

\@ifundefined{chaptermark}{%
\newcommand{\sectionmark}[1]{\markright{\MakeUppercase{\thesection.\ #1}}}
}{%
\renewcommand{\sectionmark}[1]{\markright{\MakeUppercase{\thesection.\ #1}}}
}

\@ifundefined{TSR}{}{%
\renewcommand{\py@HeaderFamily}{\rmfamily\bfseries}
\definecolor{TitleColor}{rgb}{0,0,0}
\renewcommand{\appendix}{\par
  \setcounter{section}{0}%
  \inapptrue%
  \renewcommand\thesection{\@Alph\c@section}}
}

\makeatother

% Make the pages always arabic, and don't do any pages without page
% numbers
\pagenumbering{arabic}
\pagestyle{plain}
'''

latex_logo = os.path.join(__dir__, 'latex', 'stsci_logo.pdf')

latex_additional_files = glob.glob(os.path.join(__dir__, 'latex', '*'))

# ----------------------------------------------------------------------
# GRAPHVIZ OPTIONS
# ----------------------------------------------------------------------

# To get graphviz working in our environment
graphviz_web_image_format = 'png'
graphviz_latex_image_mode = 'ps2:pdf'



from __future__ import division # confidence high
import os
import shutil

import distutils.extension
pkg = ["stsci", "stsci.sphinxext"]

setupargs = {
    'version' :         '1.2',
    'description' :
        'A set of tools and templates to customize Sphinx for use in STScI projects',

    'long_description' :
'''In short, you use this package by adding 'from stsci_docs.conf
import *' to the top of your conf.py in your Sphinx documentation
source tree.  In long, see the README file.''',

    'author' :          'Michael Droettboom',

    'author_email' :    'mdroe@stsci.edu',

    'url' :             '',

    'scripts' :         [],

    'license' :         'BSD',

    'platforms' :       ["Linux", "Solaris", "Mac OS X", "Win"],

    'package_dir':      { 'stsci' : 'lib/stsci', 'stsci.sphinxext' : 'lib/stsci/sphinxext'  },

    # how to install your data files:
    #   [
    #       ( directory_name_files_go_to, [ file_name_in_source_tree, another_data_file, etc ] )
    #   ]
    'data_files' :      [
                ( 'stsci/sphinxext/stsci_sphinx_theme', [ 'lib/stsci/sphinxext/stsci_sphinx_theme/theme.conf' ] ),
                ( 'stsci/sphinxext/stsci_sphinx_theme/static', [ 'lib/stsci/sphinxext/stsci_sphinx_theme/static/*' ] ),
                ( 'stsci/sphinxext/latex', [ 'lib/stsci/sphinxext/latex/*' ] ),
                ],

    'package_data' :      {
                'stsci.sphinxext': [ 'stsci_sphinx_theme/theme.conf', 'stsci_sphinx_theme/static/*', 'latex/*' ] }
}



"""
A custom directive to produce an abstract.

It takes an author and date as options.  These are printed above the
abstract in HTML, but not printed at all in LaTeX, since the author
and date are included by another mechanism in LaTeX.

Example::

  .. abstract::
     :author: Roy G. Biv
     :date: January 1, 2000

     This paper is about the interesting properties of colors.
"""

from docutils.parsers.rst.directives.admonitions import BaseAdmonition
from docutils.parsers.rst import directives
from docutils.nodes import Admonition, Element, title

class abstract(Admonition, Element):
    pass

class AbstractDirective(BaseAdmonition):
    node_class = abstract

    option_spec = {
        'author': directives.unchanged,
        'date'  : directives.unchanged
        }

    def run(self):
        nodes = BaseAdmonition.run(self)
        nodes[0].options = self.options
        return nodes

def visit_abstract_node_html(self, node):
    if 'author' in node.options:
        self.body.append('<p><b>Author:</b> %s</p>' % node.options['author'])
    if 'date' in node.options:
        self.body.append('<p><b>Date:</b> %s</p>' % node.options['date'])

    self.body.append(self.starttag(node, 'div', CLASS=('admonition abstract')))
    node.insert(0, title('abstract', 'Abstract'))
    self.set_first_last(node)

def depart_abstract_node_html(self, node):
    self.depart_admonition(node)

def visit_abstract_node_latex(self, node):
    self.body.append("\n\\begin{abstract}\n")

def depart_abstract_node_latex(self, node):
    self.body.append("\n\\end{abstract}\n")

def visit_abstract_node_text(self, node):
    self.new_state(2)

def depart_abstract_node_text(self, node):
    self.end_state(first="Abstract: ")

def setup(app):
    app.add_node(abstract,
                 html=(visit_abstract_node_html, depart_abstract_node_html),
                 latex=(visit_abstract_node_latex, depart_abstract_node_latex),
                 text=(visit_abstract_node_text, depart_abstract_node_text))

    app.add_directive('abstract', AbstractDirective)



"""
Adds HTML id's to the parameter names in Numpy-formatted HTML.
"""

from docutils import nodes

def setup(app):
    app.connect("doctree-resolved", add_parameter_anchors)

class Visitor(nodes.NodeVisitor):
    def default_visit(self, node):
        pass

    def unknown_visit(self, node):
        pass

    def visit_field(self, node):
        if (type(node[0]) == nodes.field_name and
            node[0].astext().strip().lower() == 'parameters'):
            field_body = node[1]
            for subnode in field_body:
                parnode = subnode[0]
                if isinstance(parnode, nodes.strong):
                    name = parnode.astext()
                    parnode['ids'].append(name)

def add_parameter_anchors(app, doctree, fromdocname):
    doctree.walk(Visitor(doctree))




"""This is an automatically generated file created by stsci.distutils.hooks.version_setup_hook.
Do not modify this file by hand.
"""

__all__ = ['__version__', '__vdate__', '__svn_revision__', '__svn_full_info__',
           '__setup_datetime__']

import datetime


__version__ = '1.2.1'
__vdate__ = 'unspecified'
__svn_revision__ = '3398'
__svn_full_info__ = 'Path: stsci_sphinxext\nURL: https://svn6.assembla.com/svn/astrolib/branches/release_2013-03/stsci_sphinxext\nRepository Root: https://svn6.assembla.com/svn/astrolib\nRepository UUID: 90a0a646-be8a-0410-bb88-9290da87bc01\nRevision: 3398\nNode Kind: directory\nSchedule: normal\nLast Changed Author: stsci_sienkiew\nLast Changed Rev: 3398\nLast Changed Date: 2013-05-01 14:14:25 -0400 (Wed, 01 May 2013)'
__setup_datetime__ = datetime.datetime(2013, 5, 1, 15, 26, 47, 840701)


def update_svn_info():
    """Update the SVN info if running out of an SVN working copy."""

    import os
    import string
    import subprocess

    global __svn_revision__
    global __svn_full_info__

    # Wind up the module path until we find the root of the project
    # containing setup.py
    path = os.path.abspath(os.path.dirname(__file__))
    dirname = os.path.dirname(path)
    setup_py = os.path.join(path, 'setup.py')
    while path != dirname and not os.path.exists(setup_py):
        path = os.path.dirname(path)
        dirname = os.path.dirname(path)
        setup_py = os.path.join(path, 'setup.py')

    try:
        pipe = subprocess.Popen(['svnversion', path],
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        if pipe.wait() == 0:
            stdout = pipe.stdout.read().decode('latin1').strip()
            if stdout and stdout[0] in string.digits:
                __svn_revision__ = stdout
    except OSError:
        pass

    try:
        pipe = subprocess.Popen(['svn', 'info', path],
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        if pipe.wait() == 0:
            lines = []
            for line in pipe.stdout.readlines():
                line = line.decode('latin1').strip()
                if not line:
                    continue
                lines.append(line)

            if not lines:
                __svn_full_info__ = 'unknown'
            else:
                __svn_full_info__ = '\n'.join(lines)
    except OSError:
        pass


update_svn_info()
del update_svn_info



"""
This extension exists only to tweak a docutils setting
`field_name_limit`.  `field_name_limit` is the number of characters
allowed in the field name before they are given their own row.  In the
Numpy documentation standard, "field names" are the sections headers
like "Parameters:" or "Returns:".  By setting `field_name_limit` to 1,
all of these section headers will be given their own line, those not
wasting so much horizontal space.

This extension is a little convoluted, since Sphinx doesn't appear to
provide a way to set docutils settings from the conf.py, so we need to
wait until we get a docutils builder (the builder_inited event), and
then tweak the setting directly.
"""

def setup(app):
    app.connect("builder-inited", builder_inited)

def builder_inited(app):
    app.builder.env.settings['field_name_limit'] = 1
